package jess;

import java.util.Map;

/**
 * A test that always passes, but makes calls with calltype LEFT
 * instead of RIGHT.
 * <P>
 * (C) 2005 Sandia National Laboratories<br>
 */

class Node1RTL extends Node1 implements LogicalNode {

    /**
     * Tokens that give logical support to facts
     */

    private NodeLogicalDependencyHandler m_logicalDepends;
    private int m_tokenSize;

    void callNodeRight(int tag, Token t, Context context) throws JessException {
        if (m_logicalDepends != null)
            m_logicalDepends.tokenMatched(tag, t, context);

        passAlong(tag, t, context);
        return;
    }


    void passAlong(int tag, Token t, Context context) throws JessException {
        m_tokenSize = t.size();
        Node[] sa = m_succ;
        for (int j = 0; j < m_nSucc; j++) {
            Node s = sa[j];
            // System.out.println(this + " " +  t);
            s.callNodeLeft(tag, t, context);
        }
    }

    public boolean equals(Object o) {
        return (o instanceof Node1RTL);
    }

    public String toString() {
        return "[Left input adapter]";
    }

    public void dependsOn(Fact f, Token t) {
        ensureHandlerAllocated();
        m_logicalDepends.dependsOn(f, t);
    }

    public int getTokenSize() {
        return m_tokenSize;
    }

    // For testing only
    public Map getLogicalDependencies() {
        return m_logicalDepends.getMap();
    }

    public void setMatchInfoSource(MatchInfoSource source) {
        ensureHandlerAllocated();
        m_logicalDepends.setMatchInfoSource(source);
    }

    private void ensureHandlerAllocated() {
        if (m_logicalDepends == null)
            m_logicalDepends = new NodeLogicalDependencyHandler(getTokenSize());
    }
}

