package jess;

import java.lang.reflect.*;

class SerializableVD implements SerializableD {
    private String m_fieldName, m_className;
    private transient Field m_field;

    SerializableVD(Class c, Field f, boolean abortOnStatic) throws JessException {
        if (abortOnStatic)
            abortIfStaticField(f);
        m_field = f;
        m_fieldName = f.getName();
        m_className = c.getName();
    }

    public String getName() {
        return m_fieldName;
    }

    public Class getPropertyType(Rete engine) throws JessException {
        if (m_field == null)
            reload(engine);
        return m_field.getType();
    }

    private void reload(Rete engine) throws JessException{
        try {
            Class c = engine.findClass(m_className);
            Field field = c.getField(m_fieldName);
            abortIfStaticField(field);
            m_field = field;

        } catch (Exception e) {
            throw new JessException("SerializableVD.reload",
                                    "Can't recreate property", e);
        }
    }

    private void abortIfStaticField(Field field) throws JessException {
        if (Modifier.isStatic(field.getModifiers()))
            throw new JessException("SerializableVD.reload",
                                    "Field is static", field.getName());
    }

    public Object getPropertyValue(Rete engine, Object o) throws JessException, IllegalAccessException {
        if (m_field == null)
            reload(engine);
        return m_field.get(o);
    }

    public void setPropertyValue(Rete engine, Object om, Object value) throws JessException, IllegalAccessException, InvocationTargetException {
        if (m_field == null)
            reload(engine);
        m_field.set(om, value);
    }
}
