package jess.xml;

import jess.JessException;
import jess.Rete;
import org.xml.sax.*;

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

/**
 A SAX-based parser for JessML. Construct a JessSAXParser with a
 Rete object as an argument, then pass an InputSource to the parse()
 method. All the constructs and other code in the file will be
 installed into the Rete object.
 <pre>
 JessSAXParser parser = new JessSAXParser(engine);
 parser.parse(new org.xml.sax.InputSource(new FileInputStream(filename));
 </pre>
 (C) 2005 Sandia National Laboratories<br>
 @see Rete#batch
 */

public class JessSAXParser {

    SAXParser m_parser;
    JessSAXHandler m_handler;

    public JessSAXParser(Rete sink) throws JessException {
        m_handler = new JessSAXHandler(sink);
        try {
            m_parser = SAXParserFactory.newInstance().newSAXParser();
        } catch (Exception e) {
            throw new JessException("JessSAXParser", "Can't initialize", e);
        }
    }

    public void parse(InputSource source) throws JessException {
        try {
            m_parser.parse(source, m_handler);
        } catch (SAXException e) {
            if (e.getException() instanceof JessException)
                throw (JessException) e.getException();
            else {
                throw new JessException("JessSAXParser.parse", "Parse error: " + e.getMessage(), e.getException());
            }
        } catch (Exception e) {
            throw new JessException("JessSAXParser.parse", "Parse error: " + e.getMessage(), e);
        }
    }
}
